@@ -1,7 +1,7 @@
 import logging
 
 from asyncio import AbstractEventLoop
-from collections import ChainMap, UserDict
+from collections import ChainMap
 from functools import wraps
 from motor.motor_asyncio import AsyncIOMotorCollection, AsyncIOMotorClient
 
@@ -10,22 +10,6 @@ from .document_dict import DocumentDict
 LOG = logging.getLogger("mdocument")
 
 
-class OnDeleteAction:
-    pass
-
-
-class DeleteDocument(OnDeleteAction):
-    pass
-
-
-class DeleteField(OnDeleteAction):
-    pass
-
-
-class PopFromArray(OnDeleteAction):
-    pass
-
-
 def check_setup(func):
     """Decorator for checking that document have valid loop and client."""
 
@@ -33,6 +17,7 @@ def check_setup(func):
         if not hasattr(cls, "_client_") and "client" not in kwargs:
             raise ClientNotFound()
         return await func(cls, *args, **kwargs)
+
     return arg_wrap
 
 
@@ -58,7 +43,7 @@ class MetaDocument(type):
     def collection(cls) -> AsyncIOMotorCollection:
         for field in ("_client_", "_database_", "_collection_"):
             if not hasattr(cls, field):
-                raise type("{}NotFound".format(field.capitalize()), (DocumentException, ), {})(
+                raise type("{}NotFound".format(field.capitalize()), (DocumentException,), {})(
                     f"Required attribute {field} is missing."
                 )
 
@@ -74,7 +59,7 @@ class Document(metaclass=MetaDocument):
 
     def __init__(self, **kwargs):
         self._document_ = DocumentDict(kwargs)
-        self._shadow_copy_ = kwargs.copy()
+        self._shadow_copy_ = DocumentDict(kwargs.copy())
         self._relations_ = {}
 
     @classmethod
@@ -144,7 +129,7 @@ class Document(metaclass=MetaDocument):
         for prop in property_data:
             self_field = prop["self_field"]
             other_field = prop["other_field"]
-            document = prop["document_func"](self)
+            document = prop["other_document_func"]()
             if self[self_field] != self._shadow_copy_[self_field]:
                 await document.collection.update_many(
                     {other_field: self._shadow_copy_[self_field]},
@@ -156,8 +141,7 @@ class Document(metaclass=MetaDocument):
         """Force update document."""
 
         await self._update_related()
-
-        return await self.collection.replace_one({"_id": self._id}, self._document_)
+        return await self.collection.replace_one({"_id": self._shadow_copy_._id}, self._document_)
 
     @classmethod
     @check_setup
@@ -196,50 +180,17 @@ class Document(metaclass=MetaDocument):
         await cls.collection.insert_one(kwargs)
         return await cls._async_init(_id=kwargs["_id"])
 
-    async def _delete_related_pop(self, document, self_field, other_field):
-        """Pop self key from other documents Array field."""
-
-        await document.collection.update_many(
-            {other_field: self[self_field]},
-            {"$pull": {other_field: self[self_field]}})
-
-    async def _delete_other(self, doc, delete_info):
-
-        if delete_info["on_delete"] is DeleteDocument:
-            for doc in await doc.many(**{
-                delete_info["self_field"]: self[delete_info["other_field"]]
-            }):
-                await doc.delete()
-        elif delete_info["on_delete"] is PopFromArray:
-            await self._delete_related_pop(document=doc,
-                                           self_field=delete_info["self_field"],
-                                           other_field=delete_info["other_field"])
-
-        elif delete_info["on_delete"] is DeleteField:
-            await doc.collection.update_many(
-                {delete_info["self_field"]: self[delete_info["other_field"]]},
-                {"$unset": {delete_info["self_field"]: ""}})
 
     async def _delete_by_prop(self, relations):
 
         for prop, delete_info in relations.items():
 
             property_name = prop.fget.__name__
-            document = delete_info["other_document"]()
-
-            if delete_info["on_delete"] is DeleteDocument:
+            try:
                 for doc in await getattr(self, property_name):
                     await doc.delete()
-
-            elif delete_info["on_delete"] is PopFromArray:
-                await self._delete_related_pop(document=document,
-                                               self_field=delete_info["self_field"],
-                                               other_field=delete_info["other_field"])
-
-            elif delete_info["on_delete"] is DeleteField:
-                await document.collection.update_many(
-                    {delete_info["other_field"]: self[delete_info["self_field"]]},
-                    {"$unset": {delete_info["other_field"]: ""}})
+            except DocumentDoesntExist:
+                continue
 
     async def _delete_related(self):
         """Deletes related documents or pops field values."""
@@ -247,21 +198,11 @@ class Document(metaclass=MetaDocument):
         self_properties = set({v for v in self.__class__.__dict__.values()
                                if isinstance(v, property)})
 
-        other_properties = {
-            k: v for k, v in Document._relations_.items() if v["other_document"]() is
-                self.__class__}
-
         relations = set(Document._relations_).intersection(self_properties)
 
         await self._delete_by_prop({prop: delete_info for prop, delete_info
-            in Document._relations_.items() if prop in relations
-        })
-
-        for prop, delete_info in other_properties.copy().items():
-
-            doc = delete_info["self_document"]()
-
-            await self._delete_other(doc, delete_info)
+                                    in Document._relations_.items() if prop in relations
+                                    })
 
     async def delete(self):
         """Delete current document and related fields or documents base on related."""
@@ -271,10 +212,11 @@ class Document(metaclass=MetaDocument):
         return result
 
     @staticmethod
-    def related(self_path, other_path, multiple=True, on_delete=None):
+    def related(self_path, other_path, multiple=True, parent=True):
         """Decorator for related documents.
 
-        :param on_delete: actions to take when document is deleted
+        :param parent: show relations type. When Parent updated or deleted Child is also updated
+        and deleted. When Child is updated or deleted Parent stays the same.
         :param self_path: Document.key to self pk
         :param other_path: Document.key to other pk
         :param multiple: return multiple documents or only one
@@ -285,11 +227,6 @@ class Document(metaclass=MetaDocument):
             self_field = ".".join(self_path.split(".")[1:])
             other_field = ".".join(other_path.split(".")[1:])
 
-            def get_self_document():
-                for subclass in Document.__subclasses__():
-                    if subclass.__name__ == self_path.split(".")[0]:
-                        return subclass
-
             def get_other_document():
                 for subclass in Document.__subclasses__():
                     if subclass.__name__ == other_path.split(".")[0]:
@@ -309,12 +246,12 @@ class Document(metaclass=MetaDocument):
             delete_item = {
                 "self_field": self_field,
                 "other_field": other_field,
-                "on_delete": on_delete,
-                "self_document": get_self_document,
-                "other_document": get_other_document
+                "parent": parent,
+                "other_document_func": get_other_document
             }
 
             Document._relations_[result] = delete_item
 
             return result
+
         return func_wrapper
